import QtQuick 2.15
import QtGraphicalEffects 1.0
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import QtQuick.Layouts 1.15
import org.ukui.quick.items 1.0 as UkuiItems
import TitleLessWindow 1.0 as MyWd
import ContentMenu 1.0

MyWd.DeactiveWindow {
    id: clipboardWindow
    visible: false
    width: 568
    height: 412
    // 设置窗口背景颜色为透明，以便于显示圆角效果
    color: "transparent"
    Connections {
        target: clipboardWindow
        function onWindowDeactivated() {
            console.log("clipboardWindow deactivated!")
            // 在这里处理窗口失活逻辑
            clipboardWindow.visible = false
        }
    }
    onVisibleChanged: {
        if (visible) {
            var pos = backend.getGlobalMousePosition()
            clipboardWindow.x = pos.x - clipboardWindow.width/2
            clipboardWindow.y = pos.y - clipboardWindow.height/2
        }
    }


    Rectangle {
        id: menuBack
        anchors.centerIn: parent
        width: clipboardWindow.width
        height: clipboardWindow.height
        color: Qt.rgba(255, 255, 255, 1)
        clip : true

        property int nowCurrentType: backend.displayedType
        property string restoreIconName: nowCurrentType === ContentType.Image ? "image-x-generic-symbolic" : "ukui-idtext-symbolic"

        Row {
            id: topLine
            spacing: 0
            Layout.alignment: Qt.AlignRight
            anchors.top: parent.top
            anchors.right: parent.right
            topPadding: 4
            rightPadding: 8
            Rectangle {
                id: restoreBtn
                width: 28
                height: 28
                radius: width / 2
                border.width: 1
                border.color: Qt.rgba(255, 255, 255, 0.05)
                color: Qt.rgba(55/255, 144/255, 250/255, 0.8)

                // 深色主题下反色
                UkuiItems.Icon {
                    height: 14
                    width: 14
                    anchors.centerIn: parent
                    mode: UkuiItems.Icon.ForceHighlight
                    source: backend.getIcon(menuBack.restoreIconName);
                }
                MouseArea {
                    id: restoreBtnArea
                    anchors.fill: parent
                    hoverEnabled: true

                    onEntered: {
                        restoreBtn.color = Qt.rgba(55/255, 144/255, 250/255, 1)
                    }

                    onExited: {
                        restoreBtn.color = Qt.rgba(55/255, 144/255, 250/255, 0.8)
                    }

                    onClicked: {
                        var currentType = backend.originalType
                        if (currentType === menuBack.nowCurrentType){
                            return
                        }
                        menuBack.nowCurrentType = currentType;
                        if (currentType === ContentType.PlainText) {
                            stack.replace(stack.currentItem, plainTextComponent.createObject())
                        } else if (currentType === ContentType.RichText) {
                            stack.replace(stack.currentItem, richTextComponent.createObject())
                        } else if (currentType === ContentType.Image){
                            stack.replace(stack.currentItem, imageComponent.createObject())
                        }
                    }
                }
            }


            Rectangle {
                id: menuBtn
                width: 20
                height: 28
                color: "transparent"

                // 深色主题下反色
                UkuiItems.Icon {
                    height: 16
                    width: 16
                    anchors.centerIn: parent
                    source: backend.getIcon("ukui-down-symbolic");
                    rotation: buttonMenu.visible ? 180 : 0
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        buttonMenu.open()
                    }
                }

                // 关联菜单
                Menu {
                    id: buttonMenu
                    width: 120
                    y: 42
                    Repeater {
                        id: control
                        model: backend.contentMenus
                        delegate: MenuItem {
                            text: modelData.itemName
                            onTriggered: {
                                var currentType = backend.contentMenus[index].contentType
                                if (currentType === menuBack.nowCurrentType){
                                    return
                                }

                                menuBack.nowCurrentType = currentType;
                                if (currentType === ContentType.PlainText) {
                                    stack.replace(stack.currentItem, plainTextComponent.createObject())
                                } else if (currentType === ContentType.RichText) {
                                    stack.replace(stack.currentItem, richTextComponent.createObject())
                                } else if (currentType === ContentType.Image){
                                    stack.replace(stack.currentItem, imageComponent.createObject())
                                }
                            }
                        }
                    }

                    function removeItem() {
                        for (var i = 0; i < control.model.length; i++) {
                             if(control.model[i].contentType === ContentType.Original) {
                                 backend.removeContentMenu(i); // 删除项
                                 break; // 只删除第一个匹配的项
                             }
                        }
                    }

                    Component.onCompleted: {
                        removeItem();
                        var type = backend.displayedType
                        if (type === ContentType.PlainText) {
                            stack.replace(stack.currentItem, plainTextComponent.createObject())
                        } else if (type === ContentType.RichText) {
                            stack.replace(stack.currentItem, richTextComponent.createObject())
                        } else if (type === ContentType.Image){
                            stack.replace(stack.currentItem, imageComponent.createObject())
                        }
                    }
                }
            }

        }

        StackView {
            id: stack
            replaceEnter: null
            replaceExit: null
            anchors.top: topLine.bottom
            anchors.bottom: buttomLine.top
            anchors.left: parent.left
            anchors.right: parent.right
        }

        Row {
            id: buttomLine
            spacing: 0
            Layout.alignment: Qt.AlignRight
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            rightPadding: 8
            bottomPadding: 12
            Label {
                id: cancelButton
                width: 41
                height: 32
                Text {
                    text: "X"
                    anchors.fill: parent
                    elide: Text.ElideRight
                    textFormat: Text.RichText
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignHCenter
                }
                background: Rectangle {
                    width: cancelButton.width-4
                    height: cancelButton.height-4
                    anchors.centerIn: parent
                    radius: 5
                    color: Qt.rgba(255, 255, 255, 1)
                    layer.enabled: true;
                    layer.effect: DropShadow {
                        radius: 5
                        samples: 25;
                    }
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        clipboardWindow.visible = false
                    }
                }
            }

            Label {
                id: pasteButton
                width: 41
                height: 32
                Text {
                    text: "\u2713"
                    anchors.fill: parent
                    elide: Text.ElideRight
                    textFormat: Text.RichText
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignHCenter
                }
                background: Rectangle {
                    width: pasteButton.width-4
                    height: pasteButton.height-4
                    anchors.centerIn: parent
                    radius: 5
                    color: Qt.rgba(255, 255, 255, 1)
                    layer.enabled: true;
                    layer.effect: DropShadow {
                        radius: 5
                        samples: 25;
                    }
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        clipboardWindow.visible = false
                        var currentType = menuBack.nowCurrentType
                        backend.copyToClipboard(currentType)
                        backend.performPasteOperation()
                        backend.pasteClicked()
                    }
                }
            }
        }

        Component {
            id: richTextComponent // 富文本项的Component
            Rectangle {
                color: Qt.rgba(255, 255, 255, 0.75)

                TextEdit {
                    anchors.fill: parent
                    focus: true
                    wrapMode: TextEdit.Wrap
                    text: backend.richText
                    textFormat: Text.RichText
                    leftPadding: 8
                    onTextChanged: {
                        backend.setRichText(text)
                    }
                }
            }
        }

        Component {
            id: imageComponent // 图片项的Component
            ScrollView {
                id: scrollView
                clip: true
                ScrollBar.horizontal.policy: scrollView.width < scrollView.contentWidth ? ScrollBar.AlwaysOn : ScrollBar.AlwaysOff
                ScrollBar.vertical.policy: scrollView.height < scrollView.contentHeight ? ScrollBar.AlwaysOn : ScrollBar.AlwaysOff
                property int columns: backend.imageNumber() > 3 ? 4 : backend.imageNumber()

                function getImageSize()
                {
                    var max_height = 330
                    var max_width = 544
                    var imageWidth = backend.imageSize.width
                    var imageHeight = backend.imageSize.height
                    var thumbHeight = max_height
                    var thumbWidth = Math.floor(max_height * imageWidth / imageHeight)
                    if (thumbWidth > max_width) {
                        thumbWidth = max_width
                        thumbHeight = Math.floor(max_width * imageHeight / imageWidth)
                    }
                    return Qt.size(thumbWidth, thumbHeight)
                }

                GridLayout {
                    anchors.fill: parent
                    columns: scrollView.columns
                    rowSpacing: 10
                    columnSpacing: 10

                    Repeater {
                        id: imageRepeater
                        model: backend.imageNumber()

                        delegate: Rectangle {
                            width: scrollView.width
                            height: scrollView.height
                            color: "transparent"
                            Image {
                                id: screenshot
                                cache: false
                                anchors.centerIn: parent
                                sourceSize: scrollView.getImageSize()
                                fillMode: Image.PreserveAspectFit
                                source: "image://inMemoryImages/Image"+index
                            }
                        }
                    }
                }

            }
        }

        Component {
            id: plainTextComponent // 纯文本项的Component
            Rectangle {
                color: Qt.rgba(255, 255, 255, 0.75)

                TextEdit {
                    id: test
                    anchors.fill: parent
                    focus: true
                    wrapMode: TextEdit.Wrap
                    text: backend.plainText
                    leftPadding: 8
                    onTextChanged: {
                        backend.setPlainText(text)
                    }
                }
            }
        }

    }
}
