#ifndef CLIPBOARDBACKEND_H
#define CLIPBOARDBACKEND_H

#include <QObject>
#include <QScopedPointer>
#include <QVariant>
#include <QIcon>
#include "data.h"

class ClipboardBackend : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QVariant contentMenus READ contentMenus NOTIFY contentMenusChanged)
    Q_PROPERTY(int displayedType READ displayedType WRITE setDisplayedType NOTIFY displayedTypeChanged)
    Q_PROPERTY(int originalType READ originalType WRITE setOriginalType NOTIFY originalTypeChanged)
    Q_PROPERTY(QSize imageSize READ imageSize WRITE setImageSize NOTIFY imageSizeChanged)
    Q_PROPERTY(QString richText READ richText WRITE setRichText NOTIFY richTextChanged)
    Q_PROPERTY(QString plainText READ plainText WRITE setPlainText NOTIFY plainTextChanged)
public:
    explicit ClipboardBackend(QObject *parent = nullptr);
    QVariant contentMenus() const { return QVariant::fromValue(m_contentMenus); }

    int displayedType() { return m_displayedType; }
    Q_INVOKABLE void setDisplayedType(int type);

    int originalType() { return m_originalType; }
    Q_INVOKABLE void setOriginalType(int type);

    QSize imageSize() { return m_imageSize; }
    Q_INVOKABLE void setImageSize(QSize size);

    QString richText() const { return m_richText; }
    Q_INVOKABLE void setRichText(const QString &richText);

    QString plainText() const { return m_plainText; }
    Q_INVOKABLE void setPlainText(const QString &plainText);

    Q_INVOKABLE QIcon getIcon(QString iconName) {m_icon = QIcon::fromTheme(iconName, m_icon);
                                                 return m_icon;};

    Q_INVOKABLE int imageNumber() const;
    Q_INVOKABLE void copyToClipboard(int contentType);
    Q_INVOKABLE void performPasteOperation();
    Q_INVOKABLE QPoint getGlobalMousePosition() const;    
    Q_INVOKABLE void pasteClicked() const;
    Q_INVOKABLE void removeContentMenu(int index);

public:
    InMemoryImageProvider *getImageProvider();
    void open();

private:
    void addContentMenu(ContentMenu *menu);
    void clearContentMenu();
    void addImage(const QImage &image);
    void clearImages();
    bool processClipboardData();
    void monitorShortCut();
    void showContentMenu();

signals:
    void contentMenusChanged();
    void richTextChanged();
    void plainTextChanged();
    void displayedTypeChanged();
    void originalTypeChanged();
    void imageSizeChanged();
    void contentMenuVisibleChanged(bool visible);
    void mouseClickedOutOfWindow();

private:
    QList<QObject*> m_contentMenus;
    int m_displayedType;
    int m_originalType;
    QSize m_imageSize;
    QString m_richText;
    QString m_plainText;
    QIcon m_icon;
    // 不需要自己释放，engine 会管理释放
    InMemoryImageProvider *m_imageProvider = nullptr;
};

#endif // CLIPBOARDBACKEND_H
