#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QKeyEvent>
#include <QCommandLineOption>
#include <QCommandLineParser>

#include "qtsingleapplication.h"
#include "clipboardbackend.h"
int main(int argc, char *argv[])
{

    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);

    QtSingleApplication app(argc, argv);
    QCommandLineParser parser;
    QCommandLineOption showWorkSpaceOption("show-menu","show contentMenu");
    parser.addOption(showWorkSpaceOption);
    parser.process(app);
    if (app.isRunning())
    {
        if (parser.isSet("show-menu")) {
            app.sendMessage(QApplication::arguments().length() > 1 ? QApplication::arguments().at(1) : app.applicationFilePath());
        }
        return EXIT_SUCCESS;
    }

    QQmlApplicationEngine engine;
    qmlRegisterUncreatableType<ContentMenu>(
        "ContentMenu", 1, 0, "ContentType",
        "Not be instantiated in QML, we should only pass the enum types to QML "
        "!");
    ClipboardBackend backend;
    QObject::connect(&app, &QtSingleApplication::messageReceived, [&backend](){
        backend.open();
    });

    engine.rootContext()->setContextProperty("backend", &backend);
    engine.addImageProvider(QLatin1String("inMemoryImages"), backend.getImageProvider());
    const QUrl url(QStringLiteral("qrc:/main.qml"));
    QObject::connect(&engine, &QQmlApplicationEngine::objectCreated,
                     &app, [url](QObject *obj, const QUrl &objUrl) {
        if (!obj && url == objUrl)
            QCoreApplication::exit(-1);
    }, Qt::QueuedConnection);
    engine.load(url);
    const QUrl url1(QStringLiteral("qrc:/ClipboardWindow.qml"));
    engine.load(url1);

    return app.exec();
}
