import QtQuick 2.15
import QtGraphicalEffects 1.0
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import QtQuick.Layouts 1.15
import TitleLessWindow 1.0 as MyWd
import ContentMenu 1.0

MyWd.DeactiveWindow {
    id: contentWindow
    visible: false
    width: 140
    height: content.height + 12
    title: qsTr("Content Window")
    property bool shouldVisible: false
    // 设置窗口背景颜色为透明，以便于显示圆角效果
    color: "transparent"

    Rectangle {
        id: menuBack
        anchors.centerIn: parent
        width: contentWindow.width
        height: contentWindow.height
        color: "white"
        clip : true
        ListView {
            id: content
            x: 4
            y: 6
            width: content.contentWidth
            height: content.contentHeight
            model: backend.contentMenus
            delegate: Label {
                id: contentButton
                Text {
                    text: modelData.itemName
                    anchors.fill: parent
                    leftPadding: 20  // 设置文本缩进
                    elide: Text.ElideRight
                    textFormat: Text.RichText
                    verticalAlignment: Text.AlignVCenter
                }
                background: Rectangle {
                    id: buttonBackground
                    anchors.fill: parent
                    color: Qt.rgba(255, 255, 255, 1)
                    radius: 4
                }
                height: 32
                width: 132
                MouseArea {
                    anchors.fill: parent
                    hoverEnabled: true

                    onEntered: {
                        buttonBackground.color =  Qt.rgba(0, 0, 0, 0.05)
                    }

                    onExited: {
                        buttonBackground.color =  Qt.rgba(0, 0, 0, 0)
                    }

                    onClicked: {
                        contentWindow.visible = false
                        var currentType =  backend.contentMenus[index].contentType
                        if (currentType === ContentType.Original) {
                            backend.pasteClicked()
                        } else {
                            backend.setDisplayedType(currentType)
                            clipboardComponent.createObject()
                        }

                    }
                }
            }

            onContentHeightChanged: {
                contentWindow.height = content.contentHeight + 16
            }
        }

    }

    Connections {
        target: contentWindow
        function onWindowDeactivated() {
            console.log("contentWindow deactivated!")

            // 在这里处理窗口失活逻辑
            if (contentWindow.visible) {
                contentWindow.visible = false
            }
        }
    }
    Connections {
        target: backend
        function onContentMenuVisibleChanged(visible) {
            contentWindow.visible = visible
        }
    }

    onVisibleChanged: {
        if (visible) {
            var pos = backend.getGlobalMousePosition()
            contentWindow.x = pos.x
            contentWindow.y = pos.y
        }
    }

    Component {
        id: clipboardComponent
        ClipboardWindow {
            id: clipboardWindow
            visible: true
        }
    }
}




